@@ -1,5 +1,5 @@
 package WebService::PayPal::PaymentsAdvanced;
-$WebService::PayPal::PaymentsAdvanced::VERSION = '0.000004';
+$WebService::PayPal::PaymentsAdvanced::VERSION = '0.000005';
 use Moo;
 
 use feature qw( say state );
@@ -8,7 +8,7 @@ use Data::GUID;
 use LWP::UserAgent;
 use MooX::StrictConstructor;
 use Type::Params qw( compile );
-use Types::Standard qw( Bool InstanceOf Str );
+use Types::Standard qw( Bool HashRef InstanceOf Num Str );
 use Types::URI qw( Uri );
 use URI;
 use URI::FromHash qw( uri uri_object );
@@ -79,6 +79,12 @@ has vendor => (
     required => 1,
 );
 
+has verbose => (
+    is      => 'ro',
+    isa     => Bool,
+    default => 1,
+);
+
 sub _build_payflow_pro_uri {
     my $self = shift;
 
@@ -101,27 +107,28 @@ sub _build_payflow_link_uri {
     );
 }
 
+sub capture_delayed_transaction {
+    my $self = shift;
+
+    state $check = compile(Str);
+    my ($pnref) = $check->(@_);
+
+    return $self->post( { TRXTYPE => 'D', ORIGID => $pnref } );
+}
+
 sub create_secure_token {
     my $self = shift;
-    my $args = shift;
+
+    state $check = compile(HashRef);
+    my ($args) = $check->(@_);
 
     my $post = $self->_force_upper_case($args);
 
     $post->{CREATESECURETOKEN} = 'Y';
     $post->{SECURETOKENID} ||= Data::GUID->new->as_string;
 
-    my $content = join '&', $self->_encode_credentials,
-        $self->_pseudo_encode_args($post);
+    my $res = $self->post($post);
 
-    my $http_response
-        = $self->ua->post( $self->payflow_pro_uri, Content => $content );
-
-    my $params
-        = WebService::PayPal::PaymentsAdvanced::Response::FromHTTP->new(
-        http_response => $http_response )->params;
-
-    my $res = WebService::PayPal::PaymentsAdvanced::Response->new(
-        params => $params );
     $self->_validate_secure_token_id( $res, $post->{SECURETOKENID} );
 
     return $res;
@@ -129,11 +136,13 @@ sub create_secure_token {
 
 sub get_response_from_redirect {
     my $self = shift;
-    my %args = @_;
+
+    state $check = compile(HashRef);
+    my ($args) = $check->(@_);
 
     my $response
         = WebService::PayPal::PaymentsAdvanced::Response::FromRedirect->new(
-        %args);
+        $args);
 
     return WebService::PayPal::PaymentsAdvanced::Response->new(
         params => $response->params );
@@ -141,18 +150,36 @@ sub get_response_from_redirect {
 
 sub get_response_from_silent_post {
     my $self = shift;
-    my %args = @_;
+
+    state $check = compile(HashRef);
+    my ($args) = $check->(@_);
 
     my $response
         = WebService::PayPal::PaymentsAdvanced::Response::FromSilentPOST
-        ->new(%args);
+        ->new($args);
 
     return WebService::PayPal::PaymentsAdvanced::Response->new(
         params => $response->params );
 }
 
+sub transaction_status {
+    my $self = shift;
+
+    state $check = compile(Str);
+    my ($pnref) = $check->(@_);
+
+    return $self->post(
+        {
+            ORIGID  => $pnref,
+            TENDER  => 'C',
+            TRXTYPE => 'I',
+        }
+    );
+}
+
 sub hosted_form_uri {
     my $self = shift;
+
     state $check = compile(
         InstanceOf ['WebService::PayPal::PaymentsAdvanced::Response'] );
     my ($response) = $check->(@_);
@@ -191,6 +218,70 @@ sub hosted_form_uri {
     );
 }
 
+sub post {
+    my $self = shift;
+
+    state $check = compile(HashRef);
+    my ($post) = $check->(@_);
+
+    $post = $self->_force_upper_case($post);
+    $post->{VERBOSITY} = 'HIGH' if $self->verbose;
+
+    my $content = join '&', $self->_encode_credentials,
+        $self->_pseudo_encode_args($post);
+
+    my $http_response
+        = $self->ua->post( $self->payflow_pro_uri, Content => $content );
+
+    my $params
+        = WebService::PayPal::PaymentsAdvanced::Response::FromHTTP->new(
+        http_response => $http_response )->params;
+
+    return WebService::PayPal::PaymentsAdvanced::Response->new(
+        params => $params );
+}
+
+sub sale_from_credit_card_reference_transaction {
+    my $self = shift;
+    state $check = compile( Str, Num );
+    my ( $origid, $amount ) = $check->(@_);
+
+    return $self->post(
+        {
+            AMT     => $amount,
+            ORIGID  => $origid,
+            TENDER  => 'C',
+            TRXTYPE => 'S',
+        }
+    );
+}
+
+sub sale_from_paypal_reference_transaction {
+    my $self = shift;
+    state $check = compile( Str, Num, Str );
+    my ( $baid, $amount, $currency ) = $check->(@_);
+
+    return $self->post(
+        {
+            ACTION   => 'D',
+            AMT      => $amount,
+            BAID     => $baid,
+            CURRENCY => $currency,
+            TENDER   => 'P',
+            TRXTYPE  => 'S',
+        }
+    );
+}
+
+sub void_transaction {
+    my $self = shift;
+
+    state $check = compile(Str);
+    my ($pnref) = $check->(@_);
+
+    return $self->post( { TRXTYPE => 'V', ORIGID => $pnref } );
+}
+
 sub _validate_secure_token_id {
     my $self     = shift;
     my $res      = shift;
@@ -260,7 +351,7 @@ WebService::PayPal::PaymentsAdvanced - A simple wrapper around the PayPal Paymen
 
 =head1 VERSION
 
-version 0.000004
+version 0.000005
 
 =head1 SYNOPSIS
 
@@ -277,7 +368,6 @@ version 0.000004
         {
             AMT            => 100,
             TRXTYPE        => 'S',
-            VERBOSITY      => 'HIGH',
             BILLINGTYPE    => 'MerchantInitiatedBilling',
             CANCELURL      => 'https://example.com/cancel',
             ERRORURL       => 'https://example.com/error',
@@ -408,6 +498,11 @@ method and deal with them accordingly.  If you disable this option, you'll need
 to rely on end users to report issues which may exist within PayPal's hosted
 pages.  Defaults to C<true>.
 
+=head3 verbose
+
+C<Boolean>.  Sets C<VERBOSITY=HIGH> on all transactions if enabled.  Defaults
+to C<true>.
+
 =head2 Methods
 
 =head3 create_secure_token
@@ -422,7 +517,6 @@ L<WebService::PayPal::PaymentsAdvanced::Response> object.
         {
             AMT            => 100,
             TRXTYPE        => 'S',
-            VERBOSITY      => 'HIGH',
             BILLINGTYPE    => 'MerchantInitiatedBilling',
             CANCELURL      => 'https://example.com/cancel',
             ERRORURL       => 'https://example.com/error',
@@ -477,6 +571,58 @@ pages or redirect the user to PayPal directly in order to make a payment.
     my $response = $payments->create_secure_token(...);
     my $uri      = $payments->hosted_form_uri($response);
 
+=head3 post
+
+Generic method to post arbitrary params to PayPal.  Requires a C<HashRef> of
+parameters and returns a L<WebService::PayPal::PaymentsAdvanced::Response>
+object.  Any lower case keys will be converted to upper case before this
+response is sent.
+
+    use WebService::PayPal::PaymentsAdvanced;
+    my $payments = WebService::PayPal::PaymentsAdvanced->new(...);
+
+    my $response = $payments->post( { TRXTYPE => 'V', ORIGID => $pnref, } );
+    say $response->message;
+
+    # OR
+    my $response = $payments->post( { trxtype => 'V', origid => $pnref, } );
+
+=head3 capture_delayed_transaction( $ORIGID )
+
+Captures a sale which you have previously authorized.  Requires the ID of the
+original transaction.  Returns a response object.
+
+=head3 sale_from_credit_card_reference_transaction( $ORIGID, $amount )
+
+Process a sale based on a reference transaction from a credit card.  Requires 2
+arguments: an ORIGID from a previous credit card transaction and an amount.
+
+    use WebService::PayPal::PaymentsAdvanced;
+    my $payments = WebService::PayPal::PaymentsAdvanced->new(...);
+
+    my $response = $payments->sale_from_credit_card_reference_transaction(
+        'BFOOBAR', 1.50'
+    );
+    say $response->message;
+
+=head3 sale_from_paypal_reference_transaction( $BAID, $amount, $currency )
+
+Process a sale based on a reference transaction from PayPal.  Requires 3
+arguments: a BAID from a previous PayPal transaction, an amount and a currency.
+
+    use WebService::PayPal::PaymentsAdvanced;
+    my $payments = WebService::PayPal::PaymentsAdvanced->new(...);
+
+    my $response = $payments->sale_from_paypal_reference_transaction(
+        'B-FOOBAR', 1.50, 'USD'
+    );
+    say $response->message;
+
+=head2 void_transaction( $ORIGID )
+
+Voids a previous transaction.  Requires the ID of the transaction to void.
+Returns a response object.
+
 =head1 AUTHOR
 
 Olaf Alders <olaf@wundercounter.com>
